@@ -15,20 +15,65 @@ from .system import System
 from .utils.time import timed
 from .support.deprecated import LoudDeprecationWarning
 
-__all__ = ['KernelPolynomialMethod', 'kpm', 'kpm_cuda',
+__all__ = ['KPM', 'kpm', 'kpm_cuda', 'SpatialLDOS',
            'jackson_kernel', 'lorentz_kernel', 'dirichlet_kernel']
 
 
-class KernelPolynomialMethod:
+class SpatialLDOS:
+    """Holds the results of :meth:`KPM.calc_spatial_ldos`
+
+    It behaves like a product of a :class:`.Series` and a :class:`.StructureMap`.
+    """
+
+    def __init__(self, data, energy, structure):
+        self.data = data
+        self.energy = energy
+        self.structure = structure
+
+    def structure_map(self, energy):
+        """Return a :class:`.StructureMap` of the spatial LDOS at the given energy
+
+        Parameters
+        ----------
+        energy : float
+            Produce a structure map for LDOS data closest to this energy value.
+
+        Returns
+        -------
+        :class:`.StructureMap`
+        """
+        idx = np.argmin(abs(self.energy - energy))
+        return self.structure.with_data(self.data[idx])
+
+    def ldos(self, position, sublattice=""):
+        """Return the LDOS as a function of energy at a specific position
+
+        Parameters
+        ----------
+        position : array_like
+        sublattice : Optional[str]
+
+        Returns
+        -------
+        :class:`.Series`
+        """
+        idx = self.structure.find_nearest(position, sublattice)
+        return results.Series(self.energy, self.data[:, idx],
+                              labels=dict(variable="E (eV)", data="LDOS", columns="orbitals"))
+
+
+class KPM:
     """The common interface for various KPM implementations
-    
+
     It should not be created directly but via specific functions
     like :func:`kpm` or :func:`kpm_cuda`.
-    
+
     All implementations are based on: https://doi.org/10.1103/RevModPhys.78.275
     """
 
     def __init__(self, impl):
+        if isinstance(impl, Model):
+            raise TypeError("You're probably looking for `pb.kpm()` (lowercase).")
         self.impl = impl
 
     @property
@@ -42,7 +87,7 @@ class KernelPolynomialMethod:
 
     @property
     def system(self) -> System:
-        """The tight-binding system (shortcut for `KernelPolynomialMethod.model.system`)"""
+        """The tight-binding system (shortcut for `KPM.model.system`)"""
         return System(self.impl.system)
 
     @property
@@ -82,9 +127,9 @@ class KernelPolynomialMethod:
         num_moments : int
             The number of moments to calculate.
         alpha : array_like
-            The starting state vector of the KPM iteration. 
+            The starting state vector of the KPM iteration.
         beta : Optional[array_like]
-            If not given, defaults to :math:`\beta = \alpha`. 
+            If not given, defaults to :math:`\beta = \alpha`.
         op : Optional[csr_matrix]
             Operator in the form of a sparse matrix. If omitted, an identity matrix
             is assumed: :math:`\mu_n = <\beta|T_n(H)|\alpha>`.
@@ -155,6 +200,32 @@ class KernelPolynomialMethod:
         return results.Series(energy, ldos.squeeze(), labels=dict(variable="E (eV)", data="LDOS",
                                                                   columns="orbitals"))
 
+    def calc_spatial_ldos(self, energy, broadening, shape, sublattice=""):
+        """Calculate the LDOS as a function of energy and space (in the area of the given shape)
+
+        Parameters
+        ----------
+        energy : ndarray
+            Values for which the LDOS is calculated.
+        broadening : float
+            Width, in energy, of the smallest detail which can be resolved.
+            Lower values result in longer calculation time.
+        shape : Shape
+            Determines the site positions at which to do the calculation.
+        sublattice : str
+            Only look for sites of a specific sublattice, within the `shape`.
+            The default value considers any sublattice.
+
+        Returns
+        -------
+        :class:`SpatialLDOS`
+        """
+        ldos = self.impl.calc_spatial_ldos(energy, broadening, shape, sublattice)
+        smap = self.system[shape.contains(*self.system.positions)]
+        if sublattice:
+            smap = smap[smap.sub == sublattice]
+        return SpatialLDOS(ldos, energy, smap)
+
     def calc_dos(self, energy, broadening, num_random=1):
         """Calculate the density of states as a function of energy
 
@@ -166,7 +237,7 @@ class KernelPolynomialMethod:
             Width, in energy, of the smallest detail which can be resolved.
             Lower values result in longer calculation time.
         num_random : int
-            The number of random vectors to use for the stochastic calculation of KPM moments. 
+            The number of random vectors to use for the stochastic calculation of KPM moments.
             Larger numbers improve the quality of the result but also increase calculation time
             linearly. Fortunately, result quality also improves with system size, so the DOS of
             very large systems can be calculated accurately with only a small number of random
@@ -206,7 +277,7 @@ class KernelPolynomialMethod:
     def calc_conductivity(self, chemical_potential, broadening, temperature,
                           direction="xx", volume=1.0, num_random=1, num_points=1000):
         """Calculate Kubo-Bastin electrical conductivity as a function of chemical potential
- 
+
         The return value is in units of the conductance quantum (e^2 / hbar) not taking into
         account spin or any other degeneracy.
 
@@ -226,7 +297,7 @@ class KernelPolynomialMethod:
         volume : Optional[float]
             The volume of the system.
         num_random : int
-            The number of random vectors to use for the stochastic calculation of KPM moments. 
+            The number of random vectors to use for the stochastic calculation of KPM moments.
             Larger numbers improve the quality of the result but also increase calculation time
             linearly. Fortunately, result quality also improves with system size, so the DOS of
             very large systems can be calculated accurately with only a small number of random
@@ -246,7 +317,26 @@ class KernelPolynomialMethod:
                               labels=dict(variable=r"$\mu$ (eV)", data="$\sigma (e^2/h)$"))
 
 
-def kpm(model, energy_range=None, kernel="default", **kwargs):
+class _ComputeProgressReporter:
+    def __init__(self):
+        from .utils.progressbar import ProgressBar
+        self.pbar = ProgressBar(0)
+
+    def __call__(self, delta, total):
+        if total == 1:
+            return  # Skip reporting for short jobs
+
+        if delta < 0:
+            print("Computing KPM moments...")
+            self.pbar.size = total
+            self.pbar.start()
+        elif delta == total:
+            self.pbar.finish()
+        else:
+            self.pbar += delta
+
+
+def kpm(model, energy_range=None, kernel="default", num_threads="auto", silent=False, **kwargs):
     """The default CPU implementation of the Kernel Polynomial Method
 
     This implementation works on any system and is well optimized.
@@ -267,14 +357,25 @@ def kpm(model, energy_range=None, kernel="default", **kwargs):
         the function reconstructed from the Chebyshev series. Possible values are
         :func:`jackson_kernel` or :func:`lorentz_kernel`. The Jackson kernel is used
         by default.
+    num_threads : int
+        The number of CPU threads to use for calculations. This is automatically set
+        to the number of logical cores available on the current machine.
+    silent : bool
+        Don't show any progress messages.
 
     Returns
     -------
-    :class:`~pybinding.chebyshev.KernelPolynomialMethod`
+    :class:`~pybinding.chebyshev.KPM`
     """
-    if kernel == "default":
-        kernel = jackson_kernel()
-    return KernelPolynomialMethod(_cpp.kpm(model, energy_range or (0, 0), kernel, **kwargs))
+    if kernel != "default":
+        kwargs["kernel"] = kernel
+    if num_threads != "auto":
+        kwargs["num_threads"] = num_threads
+    if "progress_callback" not in kwargs:
+        kwargs["progress_callback"] = _ComputeProgressReporter()
+    if silent:
+        del kwargs["progress_callback"]
+    return KPM(_cpp.kpm(model, energy_range or (0, 0), **kwargs))
 
 
 def kpm_cuda(model, energy_range=None, kernel="default", **kwargs):
@@ -291,14 +392,13 @@ def kpm_cuda(model, energy_range=None, kernel="default", **kwargs):
 
     Returns
     -------
-    :class:`~pybinding.chebyshev.KernelPolynomialMethod`
+    :class:`~pybinding.chebyshev.KPM`
     """
     try:
-        if kernel == "default":
-            kernel = jackson_kernel()
+        if kernel != "default":
+            kwargs["kernel"] = kernel
         # noinspection PyUnresolvedReferences
-        return KernelPolynomialMethod(_cpp.kpm_cuda(model, energy_range or (0, 0),
-                                                    kernel, **kwargs))
+        return KPM(_cpp.kpm_cuda(model, energy_range or (0, 0), **kwargs))
     except AttributeError:
         raise Exception("The module was compiled without CUDA support.\n"
                         "Use a different KPM implementation or recompile the module with CUDA.")
@@ -339,7 +439,7 @@ def dirichlet_kernel():
     a truncated series which results in lots of oscillation in the reconstructed function.
     Therefore, this kernel should almost never be used. It's only here in case the raw
     moment values are needed for some other purpose. Note that `required_num_moments()`
-    returns `N = pi / sigma` for compatibility with the Jackson kernel, but there is 
+    returns `N = pi / sigma` for compatibility with the Jackson kernel, but there is
     no actual broadening associated with the Dirichlet kernel.
     """
     return _cpp.dirichlet_kernel()
@@ -479,4 +579,4 @@ def _kpm_python(model, energy_range=None, kernel="default", **kwargs):
     """Basic Python/SciPy implementation of KPM"""
     if kernel == "default":
         kernel = jackson_kernel()
-    return KernelPolynomialMethod(_PythonImpl(model, energy_range or (0, 0), kernel, **kwargs))
+    return KPM(_PythonImpl(model, energy_range or (0, 0), kernel, **kwargs))
