@@ -66,6 +66,11 @@ class System(Structure):
         """:class:`.Lattice` specification"""
         return Lattice.from_impl(self.impl.lattice)
 
+    @property
+    def expanded_positions(self):
+        """`positions` expanded to `hamiltonian_size` by replicating for each orbital"""
+        return self.impl.expanded_positions
+
     @property
     def hamiltonian_size(self) -> int:
         """The size of the Hamiltonian matrix constructed from this system
@@ -147,19 +152,6 @@ class System(Structure):
 
         return np.concatenate(reduced_data)
 
-    def plot(self, num_periods=1, **kwargs):
-        """Plot the structure: sites, hoppings and periodic boundaries (if any)
-
-        Parameters
-        ----------
-        num_periods : int
-            Number of times to repeat the periodic boundaries.
-        **kwargs
-            Additional plot arguments as specified in :func:`.structure_plot_properties`.
-        """
-        kwargs.setdefault("site", {}).setdefault("radius", self.lattice.site_radius_for_plot())
-        super().plot(num_periods, **kwargs)
-
 
 def structure_plot_properties(axes='xyz', site=None, hopping=None, boundary=None, **kwargs):
     """Process structure plot properties
@@ -477,9 +469,9 @@ def plot_periodic_boundaries(positions, hoppings, boundaries, data, num_periods=
     # periodic unit cells
     for level, blend in enumerate(blend_gradient, start=1):
         shift_set = _make_shift_set(boundaries, level)
-        for shift in shift_set:
-            plot_sites(positions, data, offset=shift, blend=blend, **props['site'])
-            plot_hoppings(positions, hoppings, offset=shift, blend=blend, **props['hopping'])
+        for s in shift_set:
+            plot_sites(positions, data, offset=s, **{"blend": blend, **props["site"]})
+            plot_hoppings(positions, hoppings, offset=s, **{"blend": blend, **props["hopping"]})
 
     # periodic boundary hoppings
     for level, blend in enumerate(blend_gradient, start=1):
@@ -491,8 +483,8 @@ def plot_periodic_boundaries(positions, hoppings, boundaries, data, num_periods=
             if (shift + sign * boundary.shift) not in prev_shift_set:
                 continue  # skip existing
 
-            plot_hoppings(positions, boundary.hoppings.tocoo(), offset=shift, blend=blend,
-                          boundary=(sign, boundary.shift), **props['boundary'])
+            plot_hoppings(positions, boundary.hoppings.tocoo(), offset=shift,
+                          boundary=(sign, boundary.shift), **{"blend": blend, **props["boundary"]})
 
 
 def plot_site_indices(system):
